"""Python bindings to libpostal parse_address."""
from enum import Enum
from typing import Dict, List, Optional, Tuple


class LibpostalAddressComponent(str, Enum):
    """Libpostal address component."""

    CATEGORY = "category"
    CITY = "city"
    CITY_DISTRICT = "city_district"
    COUNTRY = "country"
    COUNTRY_REGION = "country_region"
    ENTRANCE = "entrance"
    HOUSE = "house"
    HOUSE_NUMBER = "house_number"
    ISLAND = "island"
    LEVEL = "level"
    NEAR = "near"
    PO_BOX = "po_box"
    POSTCODE = "postcode"
    ROAD = "road"
    STAIRCASE = "staircase"
    STATE = "state"
    STATE_DISTRICT = "state_district"
    SUBURB = "suburb"
    UNIT = "unit"
    WORLD_REGION = "world_region"


def _parse_address(
    address: str, language: Optional[str] = None, country_code: Optional[str] = None
) -> List[Tuple[str, str]]:
    from libpypostal import _parser  # type: ignore # pylint: disable=no-name-in-module,import-outside-toplevel

    return _parser.parse_address(  # pylint: disable=c-extension-no-member
        address, language=language, country=country_code
    )


def parse_address(
    address: str, language: Optional[str] = None, country_code: Optional[str] = None
) -> Dict[str, List[str]]:
    """Parses address into components.

    Arguments:
        address: the address to parse.
        language: optional language code to help localize parsing.
        country_code: optional country code to help localize parsing.

    Returns:
        Dictionary of address components with format {<address component>: parsed value}.
        Generally, address component lists will only have one element, but there is a
        possibility of multiple matches. Address components not found in the input are
        set to empty lists.
    """
    address_component_tuples = _parse_address(
        address, language=language, country_code=country_code
    )

    parsed_address_components: Dict[str, List[str]] = {}

    for address_component_tuple in address_component_tuples:
        component_value, component_name = address_component_tuple

        if component_name in parsed_address_components:
            parsed_address_components[component_name].append(component_value)
        else:
            parsed_address_components[component_name] = [component_value]

    for libpostal_address_component in LibpostalAddressComponent:
        if libpostal_address_component.value not in parsed_address_components:
            parsed_address_components[libpostal_address_component.value] = []

    return parsed_address_components



# pylint: disable=missing-module-docstring,missing-function-docstring,
from unittest.mock import MagicMock, patch

from libpypostal import parser


@patch("libpypostal.parser._parse_address")
def test_parse_address_single_match_each_component(
    mock__parser_parse_address: MagicMock,
) -> None:
    test_address = "123 Bayes Way, Beverly Hills, CA 90210 US"
    mock__parser_parse_address.return_value = [
        ("123", "house_number"),
        ("bayes way", "road"),
        ("beverly hills", "city"),
        ("ca", "state"),
        ("90210", "postcode"),
        ("us", "country"),
    ]
    expected = {
        "house_number": ["123"],
        "road": ["bayes way"],
        "city": ["beverly hills"],
        "state": ["ca"],
        "postcode": ["90210"],
        "country": ["us"],
        "category": [],
        "city_district": [],
        "country_region": [],
        "entrance": [],
        "house": [],
        "island": [],
        "level": [],
        "near": [],
        "po_box": [],
        "staircase": [],
        "state_district": [],
        "suburb": [],
        "unit": [],
        "world_region": [],
    }

    actual = parser.parse_address(test_address)

    assert actual == expected
    mock__parser_parse_address.assert_called_once_with(
        test_address, language=None, country_code=None
    )


@patch("libpypostal.parser._parse_address")
def test_parse_address_multiple_matches_for_component(
    mock__parser_parse_address: MagicMock,
) -> None:
    test_address = "123 Bayes Way, Beverly Hills, CA 90210 California US"
    mock__parser_parse_address.return_value = [
        ("123", "house_number"),
        ("bayes way", "road"),
        ("beverly hills", "city"),
        ("ca", "state"),
        ("california", "state"),
        ("90210", "postcode"),
        ("us", "country"),
    ]
    expected = {
        "house_number": ["123"],
        "road": ["bayes way"],
        "city": ["beverly hills"],
        "state": ["ca", "california"],
        "postcode": ["90210"],
        "country": ["us"],
        "category": [],
        "city_district": [],
        "country_region": [],
        "entrance": [],
        "house": [],
        "island": [],
        "level": [],
        "near": [],
        "po_box": [],
        "staircase": [],
        "state_district": [],
        "suburb": [],
        "unit": [],
        "world_region": [],
    }

    actual = parser.parse_address(test_address)

    assert actual == expected
    mock__parser_parse_address.assert_called_once_with(
        test_address, language=None, country_code=None
    )



from setuptools import setup, Extension


ext_modules = [
    Extension(
        "libpypostal._parser",
        sources=["src/pyparser.c", "src/pyutils.c"],
        libraries=["postal"],
        include_dirs=["/usr/local/include", "src/"],
        library_dirs=["/usr/local/lib"],
        extra_compile_args=["-std=c99"],
    ),
]

setup(ext_modules=ext_modules)



"""Libpostal data utils."""
import importlib.resources
import os
import subprocess
from typing import Literal

LIBPOSTAL_DATA_DIR_ENV_NAME = "LIBPOSTAL_DATA_DIR"
DEFAULT_DATA_VERSION = "v1.0.0"


def set_data_dir_env_var(data_dir: str) -> None:
    """Sets libpostal data directory environment variable."""
    os.environ[LIBPOSTAL_DATA_DIR_ENV_NAME] = data_dir


def download_libpostal_data(
    # pylint: disable=too-many-arguments
    output_dir: str,
    file: Literal["base", "parser", "language_classifier", "all"] = "all",
    data_dir_version: str = DEFAULT_DATA_VERSION,
    data_file_version: str = DEFAULT_DATA_VERSION,
    parser_model_version: str = DEFAULT_DATA_VERSION,
    lang_class_model_version: str = DEFAULT_DATA_VERSION,
    set_env_var: bool = True,
) -> None:
    """Downloads libpostal data.

    Arguments:
        output_dir: directory to download and expand data files to.
        file: data files to download.
        data_dir_version: data directory version.
        data_file_version: data file version.
        parser_model_version: parser model version.
        lang_class_model_version: lang class model version.
        set_env_var: indicates whether LIBPOSTAL_DATA_DIR should be set.
    """
    with importlib.resources.path(
        "libpypostal", "libpostal_download_data"
    ) as script_path:
        subprocess.run(
            [
                script_path.as_posix(),
                "download",
                file,
                output_dir,
                data_dir_version,
                data_file_version,
                parser_model_version,
                lang_class_model_version,
            ],
            check=True,
        )

    if set_env_var:
        set_data_dir_env_var(output_dir)
